#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include "linear.h"
#include <time.h>
#define Malloc(type,n) (type *)malloc((n)*sizeof(type))
#define INF HUGE_VAL
#define DINF 1.0e10


void print_null(const char *s) {}

void exit_input_error(int line_num)
{
	fprintf(stderr,"Wrong input format at line %d\n", line_num);
	exit(1);
}

static char *line = NULL;
static int max_line_len;

static char* readline(FILE *input)
{
	int len;

	if(fgets(line,max_line_len,input) == NULL)
		return NULL;

	while(strrchr(line,'\n') == NULL)
	{
		max_line_len *= 2;
		line = (char *) realloc(line,max_line_len);
		len = (int) strlen(line);
		if(fgets(line+len,max_line_len-len,input) == NULL)
			break;
	}
	return line;
}

void parse_command_line(int argc, char **argv, char *input_file_name, char *test_file_name);
void read_problem(const char *filename,struct problem *pro);
void do_cross_validation();
struct parameter param;



typedef struct pair{
	int p_index; // the index of the positive instance
	int n_index; // the index of the negative instance
	double weight; // training weight of the pair
	double alpha_; // the alpha of the pair for warm-start
}PAIR;

PAIR *tr_pairs; // recording the PAIR structure in the chosen pool for training
struct feature_node **pairs; // recording the feature vector of pairs (x-y) in the chosen pool for training, and the index agrees with the coresponding PAIR in tr_pairs.

int tail_index; // recording the current number of sampled pairs 
struct feature_node **positive; // the feature vector of positive instance in the orgin data set
struct feature_node **negative; // the feature vector of negative instance in the orgin data set
char input_file_name[1024];
char weights_file_name[1024];
double normalization; // recording the denominator for weights normalization in ASCRC 
double *dec_values; // recording decision values for calculating the AUC
int num_pairs; // the number of pair budget, corresponding to the parameter B in the paper
int step_size; // the number of pairs sampled in each round, corresponding to the parameter b in the paper
int sample_method; // the sampling strategy: 0 for random, 1 for closeness, and 2 for correctness
int prob_n;
double total_iteration; //recording the number of vector-vector multiplication takes
int _max_iter; // input argument for the maximum outer iteration that svm solver takes, which has default 1000
double Gamma; // input argument for control the importance of actual/pseudo pairs
struct model* model_;
int flag_cross_validation;
int nr_fold;
int flag_warm_start; // flag for whether doing warm-start or not: 1 for doing warm-start and 0 for not to. The default is 1
int flag_detail;  // flag for whether showing the result of each round or not: 1 for showing the result of each round and 0 for showing only the result of last round. The default is 0
int flag_correction; //flag for whether doing bias correction or not: 1 for doing bias correction and 0 for not to. The default is 1
double bias;


// vector-vector subtraction: returning the sparse svm format of vector x-y
struct feature_node* sub(struct feature_node* x, struct feature_node* y){
	int i,j,len;
	i=0; j=0; len=0;
	struct feature_node* result;

	while(x[i].index!=-1 || y[j].index!=-1){
		if(x[i].index==-1){
			while(y[j].index!=-1){
				len++;
				j++;
			}
		}else if(y[j].index==-1){
			while(x[i].index!=-1){
				len++;
				i++;
			}
		}else if(x[i].index < y[j].index ){
			len++;
			i++;
		}else if(x[i].index > y[j].index){
			len++;
			j++;
		}else{
			if(x[i].value!=y[j].value)
				len++;
			i++;
			j++;
		}
	}
	result = (struct feature_node*) calloc(len+1,sizeof(struct feature_node));
	i=0; j=0; len=0;

	while(x[i].index!=-1 || y[j].index!=-1){
		if(x[i].index==-1){
			while(y[j].index!=-1){
				result[len].index = y[j].index;
				result[len].value = (-1)*y[j].value;
				len++;
				j++;
			}
		}else if(y[j].index==-1){
			while(x[i].index!=-1){
				result[len].index = x[i].index;
				result[len].value = x[i].value;
				len++;
				i++;
			}
		}else if(x[i].index < y[j].index ){
			result[len].index = x[i].index;
			result[len].value = x[i].value;
			len++;
			i++;
		}else if(x[i].index > y[j].index){
			result[len].index = y[j].index;
			result[len].value = (-1)*y[j].value;
			len++;
			j++;
		}else{
			result[len].index = x[i].index;
			result[len].value = x[i].value - y[j].value;
			if(x[i].value!=y[j].value)
				len++;
			i++;
			j++;
		}
	}
	result[len].index = -1;
	return result;
}

// The next two functions calculate the AUC value.
int compare_auc (const void * a, const void * b)
{
	int x = *(int*)a ;
	int y = *(int*)b ;
	if(dec_values[x] < dec_values[y])
		return 1;
	else 
		return -1;
}
// the array y contains the labels and the corresponding decision value is stored in dec_values
double auc(double* y, int size){
	double roc  = 0;
	int i;
	int *indices = (int*) calloc(size,sizeof(int));
	for(i = 0; i < size; ++i) indices[i] = i;

	qsort(indices,size,sizeof(int),compare_auc);

	int tp = 0,fp = 0;
	for(i = 0; i < size; i++){
		if(y[indices[i]] == 1) tp++;
		else if(y[indices[i]] == -1){
			roc += tp;
			fp++;
		}
	}

	if(tp == 0 || fp == 0)
	{
		fprintf(stderr, "warning: Too few postive true labels or negative true labels\n");
		roc = 0;
	}
	else
		roc = roc / tp / fp;

	free(indices);

	return roc;
}

// returning the feature vector of (positive) pseudo-pair of instance x, where the parameter sign is the label of x
struct feature_node* pseudo_pair(struct feature_node* x,int sign){
	int len,i;
	for(len=0;x[len].index!=-1;len++);
	struct feature_node* result = (struct feature_node*) calloc(len+1,sizeof(struct feature_node));
	if(sign==1){
		for(i=0;i<len;i++){
			result[i].index = x[i].index;
			result[i].value = x[i].value;
		}
	}else if(sign==-1){
		for(i=0;i<len;i++){
			result[i].index = x[i].index;
			result[i].value = -1*(x[i].value);
		}

	}
	result[len].index = -1;
	return result;
}

// building the model on the current chosen pool, and storing the model in the global variable "model_"
void ranklinear(){
	int i;
	struct problem prob_rank;
	prob_rank.l = tail_index;
	prob_rank.n = prob_n;
	prob_rank.bias = bias;
	prob_rank.y = (double*) calloc(prob_rank.l,sizeof(double));
	prob_rank.W = (double*) calloc(prob_rank.l,sizeof(double));
	prob_rank.start_alpha = (double*) calloc(prob_rank.l,sizeof(double));
	prob_rank.x = (struct feature_node**) calloc(prob_rank.l,sizeof(struct feature_node*));


	//setup the svm problem to be solved
	if(sample_method==0 || flag_correction==0 ){
		normalization = tail_index;
	}
	if(flag_warm_start==1){
		for(i=0;i<tail_index;i++){
			prob_rank.W[i] = (tr_pairs[i].weight)*(double)tail_index/normalization;
			prob_rank.y[i] = 1;
			prob_rank.x[i] = pairs[i]; 
			prob_rank.start_alpha[i] = tr_pairs[i].alpha_; //restore the alpha for warm-start
		}
	}else{
		for(i=0;i<tail_index;i++){
			prob_rank.W[i] = (tr_pairs[i].weight)*(double)tail_index/normalization;
			prob_rank.y[i] = 1;
			prob_rank.x[i] = pairs[i]; 
			prob_rank.start_alpha[i] = 0.0;
		}
	}

	// build the model
	model_=warm_train(&prob_rank, &param);


	// save the alpha for warm-start
	for(i=0;i<tail_index;i++)
		tr_pairs[i].alpha_ = model_->model_alpha[i];

	// accumulate the total iteration
	total_iteration += model_->iteration;

	// free the svm problem structure
	free(model_->model_alpha);
	free(prob_rank.y);
	free(prob_rank.W);
	free(prob_rank.start_alpha);
	free(prob_rank.x);

	return ;
}

// sample a pair in the unchosen pair 
// p, n : points to the variable the positive and negative indices of the selected pair are stored, respectively
// num_postive, num_negative: the total number of positive and negative instances, respectively
// Note that for pseudo-pairs either positive index or negative index is set to -1.
void get_pair(int* p, int* n, int num_positive, int num_negative){

	int repeat = 1;
	int index_p,index_n;
	int tmp_index,i;
	do{
		if(Gamma >= 1.0){ // actual pairs only
			index_p = rand()%(num_positive);
			index_n = rand()%(num_negative);
		}else if(Gamma < 0.0){ // uniform sampling
			index_p = rand()%(num_positive+1);
			index_n = rand()%(num_negative+1);
			if(index_p==num_positive && index_n==num_negative) continue;
			if(index_p==num_positive) index_p=-1;
			if(index_n==num_negative) index_n=-1;
		}else{ // consider pseudo-pairs
			if((double)rand()/RAND_MAX < Gamma){
				index_p = rand()%(num_positive);
				index_n = rand()%(num_negative);
			}else{
				tmp_index = rand()%(num_positive+num_negative);
				if(tmp_index < num_positive){
					index_p = tmp_index;
					index_n = -1;
				}else{
					index_n = tmp_index - num_positive;
					index_p = -1;
				}
			}
		}

		repeat = 0;
		for(i=0;i<tail_index;i++){
			if(tr_pairs[i].p_index==index_p && tr_pairs[i].n_index==index_n)
				repeat = 1;
		}
	}while(repeat==1);
	*p = index_p;
	*n = index_n;
	return ;
}


// the sampling pecedure called each round: 
// pro:   the bipartite ranking problem in svm format
// num_positive, num_negative:  the total number of positive and negative instances in the svm problem "pro", respectively
// size:  the number of pairs sampled in this round
// method:  the sampling strategy; 0 for random, 1 for closeness, and 2 for correctness
// The average number pairs required to pass the probability threshold is returned.
double sample_function(struct problem *pro,int method,int num_positive,int num_negative,int size){

	int index_p,index_n,end,i;
	double *tmp;
	double distance,error,p=0,n=0,e,probability;
	double update=0; // store the average number pairs required to pass the probability threshold

	end = tail_index + size;

	switch(method){
		case 0: // random sampling
			while(tail_index != end){
				get_pair(&index_p,&index_n,num_positive,num_negative);	
				tr_pairs[tail_index].p_index= index_p;  
				tr_pairs[tail_index].n_index= index_n; 
				tr_pairs[tail_index].weight=1.0; 
				tr_pairs[tail_index].alpha_=0.0; 
				tail_index++;
			}
			update=1;
			break;

		case 1: // closeness sampling
			tmp = (double*) calloc(model_->nr_class,sizeof(double));
			while(tail_index!=end){
				get_pair(&index_p,&index_n,num_positive,num_negative);	
				update+=1;
				p=0;n=0;
				if(index_p!=-1){
					predict_values(model_,positive[index_p],tmp);
					p = tmp[0]*model_->label[0];
				}
				if(index_n!=-1){
					predict_values(model_,negative[index_n],tmp);
					n = tmp[0]*model_->label[0];
				}
				total_iteration++;
				distance = fabs(p-n);
				probability = 2.0/(1.0+exp(distance));
				if( (double)rand()/RAND_MAX < probability ){
					tr_pairs[tail_index].p_index=index_p;  
					tr_pairs[tail_index].n_index=index_n;
					tr_pairs[tail_index].weight = 1.0/probability;
					normalization += 1.0/probability;
					tr_pairs[tail_index].alpha_=0.0; 
					tail_index++;
				}
			}
			update/=size;
			free(tmp);
			break;
		case 2: // correctness sampling
			tmp = (double*) calloc(model_->nr_class,sizeof(double));
			while(tail_index!=end){
				get_pair(&index_p,&index_n,num_positive,num_negative);	
				update+=1;
				p=0;n=0;
				if(index_p!=-1){
					predict_values(model_,positive[index_p],tmp);
					p = tmp[0]*model_->label[0];
				}
				if(index_n!=-1){
					predict_values(model_,negative[index_n],tmp);
					n = tmp[0]*model_->label[0];
				}
				total_iteration++;
				error = 1.0-p+n > 0 ? 1.0-p+n : 0.0;
				e = exp(error);				
				probability =  (e-1.0+1.0e-7)/(1.0+e); // a tiny value is added to avoid near zero probability
				if( (double)rand()/RAND_MAX < probability){
					tr_pairs[tail_index].p_index=index_p;  
					tr_pairs[tail_index].n_index=index_n; 
					tr_pairs[tail_index].weight = 1.0/probability;
					normalization += 1.0/probability;
					tr_pairs[tail_index].alpha_=0.0; 
					tail_index++;
				}
			}
			update/=size;
			free(tmp);
			break;

	}

	// adjust the pairs' weight, and generate the feature vectors of the pairs. 
	for(i=tail_index-size;i<tail_index;i++){	
		if(flag_correction==0)
			tr_pairs[i].weight = 1.0;

		if(tr_pairs[i].p_index==-1){ //pair (1,(0,x_j))
			pairs[i] = pseudo_pair(negative[tr_pairs[i].n_index],-1);	
			if(Gamma > 0.0 && Gamma <1.0)
				tr_pairs[i].weight *= (1-Gamma);
		}else if(tr_pairs[i].n_index==-1){ // pair (1,(x_i,0))
			pairs[i] = pseudo_pair(positive[tr_pairs[i].p_index],1);
			if(Gamma > 0.0 && Gamma <1.0)
				tr_pairs[i].weight *= (1-Gamma);
		}else{ // pair(1,(x_i,x_j))
			pairs[i] = sub(positive[tr_pairs[i].p_index],negative[tr_pairs[i].n_index]);
			if(Gamma > 0.0 && Gamma <1.0)
				tr_pairs[i].weight *= (Gamma);
		}
	}

	if(method!=0){
		free_and_destroy_model(&model_);
	}
	return update;
}

// The main procedure that solve a bipartite ranking problem by ASCRC:
// pro:   the bipartite ranking problem in svm format
// test:  the test data in svm format
// accuracy, variance, iterations, updates:  storage for the results
// fold:  indicate which fold the procedure are solving for cross validation
void active_ranker(struct problem *pro,struct problem *test,double *accuracy,double *variance,double* iterations,double* updates,int fold){
	int i,j,k,index1,index2;
	int num_positive=0;
	int num_negative=0;
	int length = (int)ceil(num_pairs/step_size);
	double tmp,up;
	int round_size=10; // the results are averaged over 10 different initial pool
	double *t;

	for(i=0;i< pro->l;i++){
		if( (pro->y)[i] > 0 )
			num_positive++;
		else
			num_negative++;
	}

	tr_pairs = (PAIR*) calloc(num_pairs,sizeof(PAIR));
	tail_index = 0;
	positive = (struct feature_node**) calloc(num_positive,sizeof(struct feature_node*));
	negative = (struct feature_node**) calloc(num_negative,sizeof(struct feature_node*));
	pairs = (struct feature_node**) calloc(num_pairs,sizeof(struct feature_node*));

	t = (double*) calloc(2,sizeof(double));
	dec_values = (double*) calloc(test->l,sizeof(double));


	index1 =0;
	index2 =0;

	for(i=0;i<pro->l;i++){
		if((pro->y)[i] > 0){
			positive[index1] = (pro->x)[i];	
			index1++;
		}else{
			negative[index2] = (pro->x)[i];
			index2++;
		}
	}

	srand((unsigned int)fold);


	for(i=0;i<round_size;i++){
		//initialization
		param.my_max_itr = _max_iter;
		tail_index = 0;
		total_iteration = 0;
		normalization = 0.0;

		// generate the initial chosen pool
		up = sample_function(pro,0,num_positive,num_negative,step_size);
		ranklinear();
		if(flag_detail==1){
			for(k=0;k<test->l;k++){
				predict_values(model_,test->x[k],t);
				dec_values[k] = t[0]*model_->label[0];
			}
			tmp = auc(test->y,test->l);
		}else
			tmp =0;

		if(sample_method==0)	
			free_and_destroy_model(&model_);
		//record results
		accuracy[0] += tmp;
		updates[0] += up;
		variance[0] += (tmp*tmp);
		iterations[0] += total_iteration;

		for(j=1 ; (j+1)*step_size < num_pairs ; j++){
			// the sampling step
			up = sample_function(pro,sample_method,num_positive,num_negative,step_size);	
			// the training step
			ranklinear();
			if(flag_detail==1){
				for(k=0;k<test->l;k++){
					predict_values(model_,test->x[k],t);
					dec_values[k] = t[0]*model_->label[0];
				}
				tmp = auc(test->y,test->l);
			}else
				tmp =0;
			if(sample_method==0)	
				free_and_destroy_model(&model_);
			// record results
			accuracy[j] += tmp;
			updates[j] += up;
			variance[j] += (tmp*tmp);
			iterations[j] += total_iteration;
		}

		//last round
		up = sample_function(pro,sample_method,num_positive,num_negative,step_size);
		param.my_max_itr = 1000;		
		ranklinear();
		for(k=0;k<test->l;k++){
			predict_values(model_,test->x[k],t);
			dec_values[k] = t[0]*model_->label[0];
		}
		tmp = auc(test->y,test->l);

		accuracy[j] += tmp;
		updates[j] += up;
		variance[j] += (tmp*tmp);
		iterations[j] += total_iteration;

		if(sample_method==0)	
			free_and_destroy_model(&model_);
		
		// free the feature vectors of chosen pairs
		for(j=0;j<tail_index;j++)
			free(pairs[j]);
	}

	//calculate the results
	for(i=0;i<length;i++){
		accuracy[i] /= (round_size);
		iterations[i] /= (round_size);
		updates[i] /= (round_size);
		variance[i] = (variance[i]/round_size) - (accuracy[i]*accuracy[i]);
	}


	free(positive);
	free(negative);
	free(tr_pairs);
	free(pairs);
	free(dec_values);
	free(t);
	
	return ;
}

int main(int argc, char **argv)
{
	char test_file_name[1024];
	const char *error_msg;
	struct problem prob;
	struct problem sub_prob;
	struct problem test_set;
	int i,j;
	int *fold;
	int *counter;
	int length;
	int index1,index2;

	// For the next four arrays, the first index indicates the fold number and the second represents the round number of ASCRC
	double** accuracy; 
	double** variance;
	double** updates; // record the average number of pairs required to pass the probability threshold
	double** iterations; // record the number of vector-vector multiplication 

	double auc,var,up,itr;

	parse_command_line(argc, argv, input_file_name, test_file_name);
	read_problem(input_file_name,&prob);
	error_msg = check_parameter(&prob,&param);


	if(error_msg)
	{
		fprintf(stderr,"ERROR: %s\n",error_msg);
		exit(1);
	}

	prob_n = prob.n;
	int len = (int)ceil(num_pairs/step_size);

	// generate bipartite ranking problems for ASCRC
	if(flag_cross_validation){
		accuracy = (double**) calloc(nr_fold,sizeof(double*));
		updates = (double**) calloc(nr_fold,sizeof(double*));
		variance = (double**) calloc(nr_fold,sizeof(double*));
		iterations = (double**) calloc(nr_fold,sizeof(double*));
		for(i=0;i<nr_fold;i++){
			accuracy[i] = (double*) calloc(len,sizeof(double));
			variance[i] = (double*) calloc(len,sizeof(double));
			updates[i] = (double*) calloc(len,sizeof(double));
			iterations[i] = (double*) calloc(len,sizeof(double));
		}

		// partition the data set for cross validation
		length = prob.l / nr_fold;
		counter = (int*) calloc(nr_fold,sizeof(int));
		for(i=0;i<nr_fold-1;i++){
			counter[i] = length;
		}
		counter[nr_fold-1] = prob.l - (nr_fold-1)*length; 

		fold = (int*) calloc(prob.l,sizeof(int));
		for(i=0;i<prob.l;i++){
			do{
				j=rand()%nr_fold;
			}while(counter[j]==0);
			fold[i] = j;
			counter[j]--;
		}

		for(i=0;i<nr_fold-1;i++){
			counter[i] = length;
		}
		counter[nr_fold-1] = prob.l - (nr_fold-1)*length; 

		for(i=0;i<nr_fold;i++){
			// generate the sub training/testing data for cross validation
			index1=0;
			index2=0;
			sub_prob.l = prob.l - counter[i];
			test_set.l = counter[i];
			sub_prob.n = prob.n;
			test_set.n = prob.n;
			sub_prob.bias = prob.bias;
			test_set.bias = prob.bias;
			sub_prob.y = (double*) calloc(sub_prob.l,sizeof(double));
			test_set.y = (double*) calloc(test_set.l,sizeof(double));
			sub_prob.x = (struct feature_node**) calloc(sub_prob.l,sizeof(struct feature_node*));
			test_set.x = (struct feature_node**) calloc(test_set.l,sizeof(struct feature_node*));

			for(j=0;j<prob.l;j++){
				if(fold[j]!=i){
					sub_prob.y[index1] = prob.y[j];
					sub_prob.x[index1] = prob.x[j];
					index1++;
				}else{
					test_set.y[index2] = prob.y[j];
					test_set.x[index2] = prob.x[j];
					index2++;
				}
			}
			active_ranker(&sub_prob,&test_set,accuracy[i],variance[i],iterations[i],updates[i],i);
			free(test_set.y);
			free(test_set.x);
			free(sub_prob.y);
			free(sub_prob.x);
		}	
	}else{
		accuracy = (double**) calloc(1,sizeof(double*));
		variance = (double**) calloc(1,sizeof(double*));
		updates = (double**) calloc(1,sizeof(double*));
		iterations = (double**) calloc(1,sizeof(double*));

		accuracy[0] = (double*) calloc(len,sizeof(double));
		variance[0] = (double*) calloc(len,sizeof(double));
		updates[0] = (double*) calloc(len,sizeof(double));
		iterations[0] = (double*) calloc(len,sizeof(double));


		read_problem(test_file_name,&test_set);

		active_ranker(&prob,&test_set,accuracy[0],variance[0],iterations[0],updates[0],0);

		free(test_set.y);
		free(test_set.x);
		free(prob.y);
		free(prob.x);
		nr_fold = 1;
	}

	//print out the results
	fprintf(stderr,"# of pairs | AUC | variance | Avg. pairs sampled | # of dot takes\n");
	for(i=0;i<len;i++){
		auc=0;
		var=0;
		up = 0;
		itr=0;
		
		if(flag_detail==0) i=len-1;

		for(j=0;j<nr_fold;j++){
			auc+= accuracy[j][i];
			var+= variance[j][i];
			up += updates[j][i];
			itr += iterations[j][i];
		}

		printf("%d\t%g\t%g\t%g\t%g\n",(i+1)*step_size,auc/nr_fold,var/nr_fold,up/nr_fold,itr/nr_fold);

	}

	destroy_param(&param);
	free(line);
	
	return 0;
}


void parse_command_line(int argc, char **argv, char *input_file_name, char *test_file_name)
{
	int i;
	void (*print_func)(const char*) = NULL;	// default printing to stdout

	// default values
	//param.solver_type = L2R_L2LOSS_SVC_DUAL;
	param.solver_type = 3;
	param.C = 0.1;
	param.eps = INF; // see setting below
	param.p = 0.1;
	param.nr_weight = 0;
	param.weight_label = NULL;
	param.weight = NULL;
	flag_cross_validation = 0;
	flag_warm_start = 1;
	flag_correction = 1;
	flag_detail = 0;
	param.my_max_itr = 1000;
	_max_iter = 1000;
	bias = -1;
	num_pairs=8000;
	step_size=100;
	sample_method=0;
	// parse options
	for(i=1;i<argc;i++)
	{
		if(argv[i][0] != '-') break;

		if(++i>=argc)
			fprintf(stderr,"ERROR\n");
		switch(argv[i-1][1])
		{
			case 'c':
				param.C = atof(argv[i]);
				param.C *= 2.0;
				break;

			case 'B':
				num_pairs = atoi(argv[i]);
				break;

			case 'b':
				step_size = atoi(argv[i]);
				break;
			case 'g':
				// Gamma=1 for pure pair
				// 0< Gamma<1 for mixed pair
				// Gamma=0 for pure point
				// Gamma < 0 for uniformly mixed.
				Gamma = atof(argv[i]);
				break;
			case 'S':
				// the sampling strategy: 0 for random, 1 for closeness, and 2 for correctness
				sample_method = atoi(argv[i]);
				break;

			case 'I':
				//the maximum outer iteration that svm solver takes, which has default 1000
				_max_iter = atoi(argv[i]);
				break;

			case 'W':
				// not doing warm-start
				flag_warm_start = 0;
				i--;
				break;
			case 'N':
				// not doing bias correction
				flag_correction = 0;
				i--;
				break;
			case 'D':
				// show the results of each round in the AS framework
				flag_detail =1 ;
				i--;
				break;
			case 'T':
				// the threshold term theta for pseudo-pairs in the CRC framework
				bias = atof(argv[i]);
				break;

			case 'w':
				++param.nr_weight;
				param.weight_label = (int *) realloc(param.weight_label,sizeof(int)*param.nr_weight);
				param.weight = (double *) realloc(param.weight,sizeof(double)*param.nr_weight);
				param.weight_label[param.nr_weight-1] = atoi(&argv[i-1][2]);
				param.weight[param.nr_weight-1] = atof(argv[i]);
				break;

			case 'v':
				flag_cross_validation = 1;
				nr_fold = atoi(argv[i]);
				if(nr_fold < 2)
				{
					fprintf(stderr,"n-fold cross validation: n must >= 2\n");
					//	exit_with_help();
				}
				break;

			default:
				fprintf(stderr,"unknown option: -%c\n", argv[i-1][1]);
				//exit_with_help();
				break;
		}
	}
	print_func = &print_null;

	set_print_string_function(print_func);

	// determine filenames

	strcpy(input_file_name, argv[i]);

	if(i<argc-1)
		strcpy(test_file_name,argv[i+1]);
	else if(flag_cross_validation==0)
	{
		fprintf(stderr,"need test file\n");
	}

	if(param.eps == INF)
	{
		switch(param.solver_type)
		{
			case L2R_LR: 
			case L2R_L2LOSS_SVC:
				param.eps = 0.01;
				break;
			case L2R_L2LOSS_SVR:
				param.eps = 0.001;
				break;
			case L2R_L2LOSS_SVC_DUAL: 
			case L2R_L1LOSS_SVC_DUAL: 
			case MCSVM_CS: 
			case L2R_LR_DUAL: 
				param.eps = 0.1;
				break;
			case L1R_L2LOSS_SVC: 
			case L1R_LR:
				param.eps = 0.01;
				break;
			case L2R_L1LOSS_SVR_DUAL:
			case L2R_L2LOSS_SVR_DUAL:
				param.eps = 0.1;
				break;
		}
	}
}

// read in a problem (in libsvm format)
void read_problem(const char *filename, struct problem *pro)
{
	FILE *fp = fopen(filename,"r");
	int i,j,k,length,index=0;
	double value;
	int max_index=0;

	if(fp == NULL)
	{
		fprintf(stderr,"can't open input file %s\n",filename);
		exit(1);
	}

	pro->l = 0;
	max_line_len = 1024;
	line = Malloc(char,max_line_len);
	while(readline(fp)!=NULL)
	{
		(pro->l)++;
		for(j=(int)strlen(line)-1;j>=0;j--){
			if(line[j]==':'){
				k = j;
				while(line[k]!=' ') k--;
				index = atoi(line+k+1);
				break;
			}
		}
		if(index > max_index)
			max_index = index;
	}
	rewind(fp);

	pro->bias=bias;
	pro->n = max_index;
	if(pro->bias>=0)
		(pro->n)++;

	pro->y = (double*)calloc(pro->l,sizeof(double));
	pro->x = (struct feature_node**)calloc(pro->l,sizeof(struct feature_node*));

	for(i=0;i<pro->l;i++)
	{
		readline(fp);
		length=0;
		for(j=0;j<(int)strlen(line);j++){
			if(line[j]==':')
				length++;
		}
		if(pro->bias >=0)
			length++;
		(pro->x)[i] = (struct feature_node*)calloc(length+1,sizeof(struct feature_node));

		(pro->y)[i] = atof(line);
		length=0;
		for(j=0;j<(int)strlen(line);j++){
			if(line[j]==':'){
				value = atof(line+j+1);
				k = j;
				while(line[k]!=' ') k--;
				index = atoi(line+k+1);
				(pro->x)[i][length].value = value;
				(pro->x)[i][length].index = index;
				length++;
			}	
		}
		if(pro->bias>=0){
			(pro->x)[i][length].value = bias;
			(pro->x)[i][length].index = max_index+1;
			length++;
		}
		(pro->x)[i][length].value = 0;
		(pro->x)[i][length].index = -1;
	}


	fclose(fp);
}
